/*

ENC28J60 angeschlossen an Port F

PF2: INT
PF4: SS
PF5: MOSI
PF6: MISO
PF7: SCK
*/


/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich, Schrder Dietmar
 Remarks:        
 known Problems: none
 Version:        07.11.2009
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "driver/ebi_driver.h"
#include "myintTypes.h"
#include "config.h"
#include "usart.h"
#include "enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "httpd.h"
#include "telnetd.h"
#include "cmd.h"
#include "ntp.h"
#include "base64.h"
#include "analog.h"
#include "driver/wdt_driver.h"
#include "mydatad.h"
#include <stdio.h>



void configureSystemClock(void){
  
  //noch luft der Chip auf 2 MHz RC


  //--- 16MHz-Quarz benutzen ---
  OSC.XOSCCTRL = OSC_FRQRANGE_12TO16_gc //Siehe iox128a1.h
               //| OSC_XOSCSEL_XTAL_1KCLK_gc //1K clk startup time
               | OSC_XOSCSEL_XTAL_16KCLK_gc
               ;

  CLK.PSCTRL = 0;
  OSC.CTRL = OSC_XOSCEN_bm; //Quarz einschalten

  while(! (OSC.STATUS & OSC_XOSCRDY_bm) ){ //Warte, bis Quarz stabil
    _delay_ms(5);
  }
  CCP = CCP_IOREG_gc; //CCP_SPM_gc; //Signatur
  CLK.CTRL = CLK_SCLKSEL_XOSC_gc; //Quarz als Takt benutzen

  //Jetzt luft der Chip auf 16 MHz Quarz

  // --- PLL konfigurieren ---
  OSC.PLLCTRL = OSC_PLLFAC1_bm | OSC_PLLSRC_XOSC_gc; // * 2 ; 11: Ext.Clock (Quarz)

  //PLL einschalten
  OSC.CTRL |= OSC_PLLEN_bm;


  //Warte, bis PLL stabil
  while(! (OSC.STATUS & OSC_PLLRDY_bm ) ){ //PLL not ready
    _delay_ms(5);
  }
  CCP = CCP_IOREG_gc; //CCP_SPM_gc; //Signatur
  CLK.CTRL = CLK_SCLKSEL_PLL_gc;  // PLL als Takt benutzen

  //Jetzt luft der Chip auf 32 MHz
}

void initExternalRam(void){
/*--------------------------------------------------------------------------------------*/
  //SRAM einbinden mittels EBI (External Bus Interface)
  //RAM an Port H:(Ctrl), 
  //            J:D0..D3, A8..A11
  //            K:A0..A7
  /*! \brief Address where we want SDRAM to be accessed. */
  #define SDRAM_ADDR 0x4000

  /*! \brief Macro to access SDRAM. */
  #define SDRAM(addr) ((uint8_t *) SDRAM_ADDR)[addr]

  /*! \brief Number of test bytes to store to SDRAM */
  #define VECTOR_SIZE 40000L
  /*
   *  Hardware setup for 3-port SDRAM interface:
   *
   *  PORTK[7:0] - A[7:0]
   *
   *  PORTJ[7:0] - {A[11:8],D[3:0]}
   *
   *  PORTH[7:0] - {WE,CAS,RAS,DQM,BA0,BA1,CKE,CLK}
   *
   */

  /* Set signals which are active-low to high value */
  PORTH.OUT = 0x0F;

  /* Configure bus pins as outputs(except for data lines). */
  PORTH.DIR = 0xFF;
  PORTK.DIR = 0xFF;
  PORTJ.DIR = 0xF0;

  /* Set direction and output value of Chip Select line (C0). */
  PORTC.DIR = 0x01;
  PORTC.OUT = 0x00;

  /* Initialize EBI. */
  EBI_Enable( EBI_SDDATAW_4BIT_gc,
              EBI_LPCMODE_ALE1_gc,
              EBI_SRMODE_ALE12_gc,
              EBI_IFMODE_3PORT_gc );

  /* Initialize SDRAM */
  EBI_EnableSDRAM( EBI_CS_ASPACE_2MB_gc, //EBI_CS_ASPACE_8KB_gc,   /* 8 KB address space. */
                   (void *) SDRAM_ADDR,    /* Base address. */
                   false,                  /* 2 cycle CAS Latency. */
                   false,                  /* 11 Row bits. */
                   EBI_SDCOL_8BIT_gc,      /* 8 Column bits. */
                   EBI_MRDLY_1CLK_gc,      /* 1 cycle Mode Register Delay. */
                   EBI_ROWCYCDLY_1CLK_gc,  /* 1 cycle Row Cycle Delay. */
                   EBI_RPDLY_1CLK_gc,      /* 1 cycle Row to Pre-charge Delay. */
                   EBI_WRDLY_1CLK_gc,      /* 1 cycle Write Recovery Delay. */
                   EBI_ESRDLY_1CLK_gc,     /* 1 cycle Exit Self Refresh to Active Delay. */
                   EBI_ROWCOLDLY_1CLK_gc,  /* 1 cycle Row to Column Delay. */
                   0x03FF,                 /* 1023 cycle Refresh Period (32.8 ms @ 2MHz). */
                   0x0100 );               /* 256 cycle Initialization Delay (128 us @ 2MHz). */

}
//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void){  
U32 lasttime=0;
U8 i;
//U16 count=0;
//U8 n;
/*
  #if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
  WDTCSR = 7;
  #else  
  WDTCR = 7; //2s Timeout
  #endif 
*/

  WDT_Reset();//Reset Watchdog
  WDT_EnableAndSetTimeout( WDT_PER_4KCLK_gc ); /* 4K cycles (4s @ 3.3V) */

  configureSystemClock();


  initExternalRam();

  //Konfiguration der Ausgnge bzw. Eingnge
  //Definition erfolgt in der config.h
  PORTA.DIR = OUTA;
  PORTC.DIR = OUTC;
  PORTD.DIR = OUTD;

  PORTA_OUTSET = 3; //LEDs an

  
  usart_init(BAUDRATE); // setup the UART
  PORTA_OUTSET = 4; //LED 3 an (Grn)
  
  
  usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
  usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");

  ADC_Init();
  
  #if defined (__atxmega128a1__)
  //  DAC_Init();
  #endif

  WDT_Reset();
  _delay_ms(800);


  //Applikationen starten
  stack_init();
  WDT_Reset();//Reset Watchdog


  httpd_init();
  telnetd_init();

  mydata_init();

 
  PORTA_OUTTGL= 2; //LED 2 aus
  
  usart_write("Enable INT\r\n");
  
  //Ethernetcard Interrupt erlauben
  ETH_INT_ENABLE;

  PORTA_OUTCLR = 7; //LEDs aus 

  //Globale Interrupts einschalten
  sei(); 


  WDT_Reset();//Reset Watchdog

  ntp_init();
  ntp_request();
  

//  PORTA_OUTTGL= 2; //LED 2 an


  //ANALOG_ON;

  usart_write("\r\nSystem Ready.\r\n");
 
  ntp_timer=30;

  while(1)
  {
    WDT_Reset();//Reset Watchdog 
    PORTA_OUTTGL =2; //LED Gelb flackern lassen
  

#if 1
    if (mydatad_status.connected){
      if (lasttime!=time){
        lasttime = time;
        mydata_write_char('*');
        for (i=0; i<50; i++){ 
          mydata_write_char(48+ time %10);
        }  
        mydata_write_char(13);
        mydata_write_char(10);
        usart_write("*\r\n");
        mydatad_send_data();
      } 
    }
#endif
  
    //usart_write("time=:%i\r\n",time);
    
    eth_get_data();
    
    //Terminalcommandos auswerten
    if (usart_status.usart_ready){
      usart_write("\r\n");
      if(extract_cmd(&usart_rx_buffer[0]))
      {
        usart_write("Ready\r\n\r\n");
      }
      else
      {
        usart_write("ERROR\r\n\r\n");
      }
      usart_status.usart_ready =0;
    }
    
#if 1
    //Empfang von Zeitinformationen
    if(!ntp_timer){
      ntp_timer = 30; //Bei korrektem Empfang wird auf NTP_REFRESH gesetzt
      ntp_request();
    }
#endif		
           
    //USART Daten fr Telnetanwendung?
    telnetd_send_data();
    

  }
    
  return(0);
}

