/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        24.10.2007
 Description:    Timer Routinen

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "timer.h"

volatile unsigned long time;
volatile unsigned long time_watchdog = 0;

//----------------------------------------------------------------------------
//Diese Routine startet und inizialisiert den Timer
void timer_init (void)
{
#if defined (__atxmega128a1__)
  TCF0.CTRLA = TC_CLKSEL_DIV1024_gc; // 32.000.000Hz / 1024 = 31250 Hz
  TCF0.CTRLB = TC_WGMODE_NORMAL_gc;
  TCF0.CTRLC = 0;
  TCF0.CTRLD = TC_EVACT_OFF_gc | TC_EVSEL_OFF_gc; //Keine Events
  TCF0.CTRLE = 0;
  TCF0.INTCTRLA = TC_OVFINTLVL_HI_gc;
  TCF0.INTCTRLB = 0; // Keine Capture/Events

  TCF0.PER = 31250-1; //gibt 1s Takt bei 32 MHz

  PMIC.CTRL |= PMIC_HILVLEN_bm;
//Interrupt geht an TCF0_OVF_vect


#else
  TCCR1B |= (1<<WGM12) | (1<<CS10 | 0<<CS11 | 1<<CS12);
	TCNT1 = 0;
	OCR1A = (F_CPU / 1024) - 1;
	#if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
  	TIMSK1 = (1 << OCIE1A);
  #else
  	TIMSK |= (1 << OCIE1A);
	#endif 

  
  //Timer0 fr die Stromverbrauchsmessung initialisieren. Erzeugt IRQs mit 20Hz
  #if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
    TCCR0A =  0; 
    TCCR0B =  0b00000101; //Vorteiler 1024 -> gibt 15625 KHz Zhlfrequenz und 61,035Hz IRQ's
  #else    
    TCCR0 =  0b00000101; //Vorteiler 1024 -> gibt 15625 KHz Zhlfrequenz und 61,035Hz IRQ's
  #endif 
  #if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
    TIMSK0 = 1<<TOIE0; //enable overflow INT 
  #else  
    TIMSK |= 1<<TOIE0; //enable overflow INT 
  #endif 
#endif 
  

  
return;
};

//----------------------------------------------------------------------------
//Timer Interrupt
#if defined (__atxmega128a1__)
ISR (TCF0_OVF_vect){
#else
ISR (TIMER1_COMPA_vect){
#endif

  PORTA_OUTTGL = 1; //LED im Sekundentakt blinken lassen

  //Sekunde um 1 erhhen
  time++;

  if((time_watchdog++) > WTT)
      {
      time_watchdog = 0;
      stack_init();
      }
  eth.timer = 1;
  ntp_timer--;
      
}

